﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using HeuristicLab.Clients.Hive.SlaveCore.Views.Properties;
using HeuristicLab.Common;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;


namespace HeuristicLab.Clients.Hive.SlaveCore.Views {

  [View("HeuristicLab Slave Main View")]
  [Content(typeof(SlaveItem), IsDefaultView = false)]
  public partial class SlaveMainViewBase : ItemView {

    public FormWindowState WindowState { get; set; }

    public event EventHandler<EventArgs<FormWindowState>> WindowStateChanged;
    public void OnWindowStateChanged(FormWindowState msg) {
      var handler = WindowStateChanged;
      if (handler != null) handler(this, new EventArgs<FormWindowState>(msg));
    }

    public new SlaveItem Content {
      get { return (SlaveItem)base.Content; }
      set {
        if (base.Content != value) {
          base.Content = value;
        }
      }
    }

    void Content_UserVisibleMessageFired(object sender, Common.EventArgs<string> e) {
      if (Settings.Default.ShowBalloonTips) {
        notifyIcon.ShowBalloonTip(2000, "HeuristicLab Hive", e.Value, ToolTipIcon.Info);
      }
    }

    public SlaveMainViewBase() {
      InitializeComponent();
      WindowState = FormWindowState.Normal;
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();

      //cSlaveView.Content = Content;
      jobsView.Content = Content;
      logView.Content = Content;
      if (Content != null) {
        Content.UserVisibleMessageFired += new System.EventHandler<Common.EventArgs<string>>(Content_UserVisibleMessageFired);
        ((SlaveItem)base.Content).Open();
        Task.Factory.StartNew(Connector);
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      //do nothing at the moment, we have nothing editable
    }

    private void notifyIcon_DoubleClick(object sender, EventArgs e) {
      if (WindowState == FormWindowState.Normal) {
        MinimizeToTray();
      } else {
        RestoreFromTray();
      }
    }

    private void closeToolStripMenuItem_Click(object sender, EventArgs e) {
      Application.Exit();
    }

    private void showToolStripMenuItem_Click(object sender, EventArgs e) {
      if (WindowState != FormWindowState.Normal) {
        RestoreFromTray();
      }
    }

    public void MinimizeToTray() {
      WindowState = FormWindowState.Minimized;
      OnWindowStateChanged(WindowState);
    }

    public void RestoreFromTray() {
      WindowState = FormWindowState.Normal;
      OnWindowStateChanged(WindowState);
    }

    private void notifyIcon_BalloonTipClicked(object sender, EventArgs e) {
      RestoreFromTray();
    }

    private void homepageToolStripMenuItem_Click(object sender, EventArgs e) {
      System.Diagnostics.Process.Start("http://dev.heuristiclab.com");
    }


    private void Connector() {
      bool connected = false;
      while (!connected) {
        this.Invoke(new Func<bool>(() => connected = ((SlaveItem)base.Content).ReconnectToSlaveCore()));

        if (!connected) {
          Thread.Sleep(1000);
        }
      }
      this.Invoke(new Action(SetEnabledStateOfControls));
    }
  }
}
